<?php

class ConnectionSpec {
    var $server;
    var $database;
    var $user;
    var $password;
    function ConnectionSpec($server, $database, $user, $password) {
        $this->server = $server;
        $this->database = $database;
        $this->user = $user;
        $this->password = $password;
    }
}

class AbstractQueryBuilder {
    var $fields = array();
    var $tables = array();
    var $values = array();
    var $aliases = array();
    var $where = array();
    var $order = array();
    function get_list($list, $delimiter) {
        return implode($delimiter, $list);
    }
    function get_value_pair($field) {
        $value = $this->get_value($field);
        return $this->get_field($field)."=".$value;
    }
    function get_value($field) {
        $value = $this->values[$field];
        if (is_string($value)) {
            $value = $this->escape_chars($value);
            $value = "\"".$value."\"";
        }
        return $value;
    }
    function get_alias_pair($field) {
        if (array_key_exists($field, $this->aliases)) {
            $alias = $this->aliases[$field];
            return $this->get_field($field)." AS ".$alias;
        }
        return $this->get_field($field);
    }
    function get_field($field) { return $field; }
    function escape_chars($value) { return $value; }
    function build_select() {}
    function build_insert() {}
    function build_update() {}
    function build_delete() {}
}

class AbstractDataSource {
    var $connectionspec;
    var $errors = array();
    var $requestdata = array();
    function AbstractDataSource($connectionspec) {
        $this->connectionspec = $connectionspec;
        $this->open();
    }
    function open() {}
    function close() {}
    function query($query) {}
    function execute($query) {}
    function clearerrors() {
        $errors = array();
    }
    function start() {}
    function commit() {}
    function rollback() {}
    function get_query_builder() {
        return new AbstractQueryBuilder();
    }
}

class MySQLQueryBuilder extends AbstractQueryBuilder {
    function get_field($field) {
        if ($field == "*") return $field;
        if (strpos($field, ".") > 0) $field = str_replace(".", ".`", $field);
        else $field = "`".$field;
        return $field."`";
    }
    function escape_chars($value) {
        return str_replace('"', '\"', $value);
    }
    function build_select() {
        $selectfields = array();
        foreach ($this->fields as $field) $selectfields[] = $this->get_alias_pair($field);
        $result = "SELECT ".$this->get_list($selectfields, ",");
        $result = $result." FROM ".$this->get_list($this->tables, ",");
        if (count($this->where) > 0) {
            $wherepairs = array();
            foreach ($this->where as $field) $wherepairs[] = $this->get_value_pair($field);
            $result = $result." WHERE ".$this->get_list($wherepairs, " AND ");
        }
        if (count($this->order) > 0) {
            $fields = array();
            foreach ($this->order as $field) $fields[] = $this->get_field($field);
            $result = $result." ORDER BY ".$this->get_list($fields, ",");
        }
        return $result;
    }
    function build_insert() {
        $result = "INSERT INTO ".$this->get_list($this->tables, ",");
        $fields = array();
        foreach ($this->fields as $field) $fields[] = $this->get_field($field);
        $result = $result." (".$this->get_list($fields, ",").")";
        $pairs = array();
        foreach ($this->fields as $field) $pairs[] = $this->get_value($field);
        $result = $result." VALUES (".$this->get_list($pairs, ",").")";
        return $result;
    }
    function build_update() {
        $result = "UPDATE ".$this->get_list($this->tables, ",");
        $valuepairs = array();
        foreach ($this->fields as $field) $valuepairs[] = $this->get_value_pair($field);
        $result = $result." SET ".$this->get_list($valuepairs, ",");
        if (count($this->where) > 0) {
            $wherepairs = array();
            foreach ($this->where as $field) $wherepairs[] = $this->get_value_pair($field);
            $result = $result." WHERE ".$this->get_list($wherepairs, " AND ");
        }
        return $result;
    }
    function build_delete() {
        $result = "DELETE FROM ".$this->get_list($this->tables, ",");
        if (count($this->where) > 0) {
            $wherepairs = array();
            foreach ($this->where as $field) $wherepairs[] = $this->get_value_pair($field);
            $result = $result." WHERE ".$this->get_list($wherepairs, " AND ");
        }
        return $result;
    }
}

class MySQLDataSource extends AbstractDataSource {
    var $connection;
    var $in_transaction;
    function MySQLDataSource($connectionspec) {
        parent::AbstractDataSource($connectionspec);
    }
    function open() {
        $this->connection = mysql_connect($this->connectionspec->server,
            $this->connectionspec->user, $this->connectionspec->password);
        if (mysql_error() != "") $this->errors[] = mysql_errno().": ".mysql_error();
        if (count($this->errors)) return;
        mysql_select_db($this->connectionspec->database, $this->connection);
    }
    function close() {
        mysql_close($this->connection);
    }
    function query($query) {
        $this->requestdata = array();
        $result = mysql_query($query);
        if (mysql_error() != "") $this->errors[] = mysql_errno().": ".mysql_error();
        if (mysql_num_rows($result) > 0) {
            while ($row = mysql_fetch_object($result))
                $this->requestdata[] = $row;
        }
        mysql_free_result($result);
    }
    function execute($query) {
        if (count($this->errors) > 0) return;
        $this->requestdata = array();
        mysql_query($query);
        if (mysql_error() != "") {
            $this->errors[] = mysql_errno().": ".mysql_error();
            $this->rollback();
        }
    }
    function start() {
        if ($this->in_transaction) return;
        mysql_query("BEGIN");
        if (mysql_error() != "") {
            $this->errors[] = mysql_errno().": ".mysql_error();
            $this->rollback();
        } else
            $this->in_transaction = true;
    }
    function commit() {
        if ($this->in_transaction) {
            mysql_query("COMMIT");
            if (mysql_error() != "") {
                $this->errors[] = mysql_errno().": ".mysql_error();
                $this->rollback();
            }
            $this->in_transaction = false;
        }
    }
    function rollback() {
        if ($this->in_transaction) {
            mysql_query("ROLLBACK");
            if (mysql_error() != "") {
                $this->errors[] = mysql_errno().": ".mysql_error();
            }
            $this->in_transaction = false;
        }
    }
    function get_query_builder() {
        return new MySQLQueryBuilder();
    }
}

?>
